/************************************************************************/
/*      Title:          fio.h                                           */
/*      Description:    This is a header file for File I/O .            */
/*      Header File Needed:                                             */
/*              portable.h                                              */
/************************************************************************/

#ifndef FIO_H
#define FIO_H

#if defined(_MSC_VER)
#define MSC
#endif 

#ifndef STD_DEF
#if defined(MSC) || defined(__WATCOMC__) || defined(WIN32)
#include        "portable.h"
#endif 
#endif  /* ifndef */


/*-----------------------------------------------------------------------
-  define constants                     
-------------------------------------------------------------------------
*/
#define dRDMODE 0       /* file if opened for read only         */
#define dWTMODE 1       /* file if opened for write only        */

#define dTGACOLOR 0x02  /* TARGA standard color image data file */
#define dTGAB_W   0x03  /* TARGA standard Black-White image data file   */
#define dJPEGCOMP 0x18  /* JPEG compressed image data file      */
#define dJPEGORIG 0x19  /* JPEG original image data file        */

#define dHDERFLEN  17   /* length of fixed part of TARGA file header */
#define dCSMSIZE   12   /* Color Space Matrix is 4 X 3         */
#define dMAX_ID_LEN 255 /* max. length of ID field             */

#define dMAX_FIO_BUF_LEN 8192   /* file I/O buffer max. length  */

#define jMAX_COMP 4     /* Max. component number-JPEG baseline  */
#define jMAX_QTBL 4     /* Max. Quantization tables             */
#define jMAX_CTBL 2     /* Max. DC or AC Huffman Code tables,   */
			/* for Baseline only system, = 2        */


/*-----------------------------------------------------------------------
-  define macros                        
-------------------------------------------------------------------------
*/

#if !defined(_WINDOWS)
#define f_r_err(Y)      { close((Y)); \
			  perror("read error\n"); \
			  return(READ_ERROR); }
#define f_w_err(W)      { close((W)); \
			  perror("write error\n"); \
			  return(WRITE_ERROR); }
#else
#define wf_r_err(hf)    { _lclose(hf); \
			  MessageBox(NULL, (LPSTR)"read error", "", \
				  MB_OK|MB_ICONSTOP); \
			  return(READ_ERROR); }
#define wf_w_err(hf)    { _lclose(hf); \
			  MessageBox(NULL, (LPSTR)"write error", "", \
				  MB_OK|MB_ICONSTOP); \
			  return(WRITE_ERROR); }
#endif /* !_WINDOWS */


/*-----------------------------------------------------------------------
-  structure definitions
-------------------------------------------------------------------------
*/

struct TARGA_FILE
{
	int fd;         /* file descriptor      */
	char FAR *fname;    /* pointer to file name string  */
	BYTE mode;      /* READ/WRITE mode indecator: 0 = read, 1 = write */
	BYTE  cmap_tp;  /* type of color map: 0 = no cmap; 1 = include cmap */
	BYTE  img_tp;   /* image type: 2 = TARGA color, 3 = TARGA B/W,  */
			/* 0x18 = compressed image, 0x19 = original image */
	UWORD cmap_st;  /* color map origine            */
	UWORD cmap_len; /* color map length             */
	BYTE  cmap_sz;  /* the size of color map (= 16 here) */
	UWORD sx;       /* X-origin of image            */
	UWORD sy;       /* Y-origin of image            */
	UWORD dx;       /* Width of image               */
	UWORD dy;       /* Height of image              */
	BYTE  pxl_sz;   /* pixel size (8 - 16, 24, 32)  */
	BYTE  img_des;  /* image descriptor :           */
			/* bit 3-0: # of attribute bits */
			/* bit 4: reserved              */
			/* bit 5: screen origine, 0 = lower left        */
			/* 1 = upper left               */
			/* bit 7-6: should be 00        */
	char  FAR *idp;   /* pointer to image ID string, used by TARGA file */
	struct JPEG_HDER FAR *jp;   /* pointer to JPEG header structure     */
#if defined(_WINDOWS)           /* WINDOWS need the Mem. Handle */
	HANDLE hid;             /* Handle for ID string         */
	HANDLE hj;              /* Handle for JPEG header       */
#endif /* _WINDOWS */
   };

struct JPEG_HDER
{
	BYTE Nver;      /* JPEG test image format version #     */
	BYTE Ctp;       /* Coding type:                                 */
			/* B0 - image type: 0 = MONO., 1 = Color        */
			/* B1 - Coder type: 0 = Huffman, 1 = Arithmetic */
			/* B2,3 - Proc. type: 00 = Extended Seq.        */
			/*                    01 = Progs. Spectr. Selec.*/
			/*                    10 = Progs. Succ. Approx. */
			/*                    11 = Full Progs.          */
			/* B4,5 - Alg. type:  00 = unused               */
			/*                    01 = Base line Alg.       */
			/*                    10 = Extended Alg.        */
			/*                    11 = Hierarchical Alg.    */
			/* B6,7 - Always set to 00                      */
	UWORD Qf;       /* Image Quality Factor         */
	BYTE Intl;      /* interleave: 0 = No interleave, Original image*/
			/*             1 = Pixel interleave;            */
			/*             2 = Component sequential;        */
	BYTE Cspc;      /* Color Space: 0 = RGB, 1 = YCbCr, 2 = YCrCb   */
			/* 3 = CMYK, 4 = YIQ, 5 = Y only, 6 = CIELUV    */
			/* 7 = CIELAB, 8 = Other                        */
	BYTE CspcM;     /* Color Space Matrix used: 0 = default;        */
			/* 1 = C.S.M. defined;  2 = No C.S.M.;          */
			/* 3 = Non Linear Color Space                   */
	UWORD RaspH;    /* Aspect Ratio Horizontal                      */
	UWORD RaspV;    /* Aspect Ratio Vertical                        */
	BYTE Ncomp;     /* N. of comp. & type of sampling:              */
			/* B0 - type of sampling: 0 = centered subsample*/
			/*                        1 = left justified    */
	BYTE HV[jMAX_COMP];     /* H. & V. sampling pixel for comp. #i  */
	INT FAR *CSMp;  /* pointer to Color Space Matrix, if present    */
};

struct F_BUF
{
	int fd;         /* file descriptor for I/O      */
	UWORD l;        /* length of this buffer        */
	BYTE rw;        /* read/write mode identifier   */
			/* 0 = r, 1 = w, r/w mode is not supported */
	UWORD sp;       /* space left in this buffer, when read */
			/* it is the data available for read,   */
			/* when write, is the space left for write      */
	BYTE FAR *hp;   /* head pointer of the buffer, fixed    */
	BYTE FAR *cp;   /* point to the current available segment       */
#if defined(_WINDOWS)
	HANDLE hM;      /* Handle to the Mem. Buffer Object, Windows only */
	HANDLE hMB;     /* Handle to the Buf. struct itself */
#endif /* _WINDOWS */
};


/*------------------------------------------------------------------------
-  Function prototypes
--------------------------------------------------------------------------
*/
/* FAR, PASCAL are defined as NULL, unless for IGV-DLL */

int MTPROCALL FIORdTGAHeader(struct TARGA_FILE FAR *);
int MTPROCALL FIOWtTGAHeader(struct TARGA_FILE FAR *);
int PASCAL FIOTypeTGAHeader(FILE FAR *, struct TARGA_FILE FAR *);
int MTPROCALL FIOFreeTGAHeader(struct TARGA_FILE FAR *);
struct F_BUF FAR * MTPROCALL FIOBufRequest(int, int, int);
int MTPROCALL FIOBufRelease(struct F_BUF FAR *);
int MTPROCALL FIOBufWrite(struct F_BUF FAR *, BYTE FAR *, int);
int MTPROCALL FIOBufRead(struct F_BUF FAR *, BYTE FAR *, int);


/***** TIFF definitions and types *****/

#define TIFF_VERSION    42

#define TIFF_BIGENDIAN          0x4d4d
#define TIFF_LITTLEENDIAN       0x4949

#define TIFFTAG_SUBFILETYPE             254     /* subfile data descriptor */
#define FILETYPE_REDUCEDIMAGE           0x1     /*reduced resolution version*/
#define FILETYPE_PAGE                   0x2     /* one page of many */
#define FILETYPE_MASK                   0x4     /* transparency mask */
#define TIFFTAG_OSUBFILETYPE            255     /* +kind of data in subfile */
#define OFILETYPE_IMAGE                 1       /*full resolution image data*/
#define OFILETYPE_REDUCEDIMAGE          2       /* reduced size image data */
#define OFILETYPE_PAGE                  3       /* one page of many */
#define TIFFTAG_IMAGEWIDTH              256     /* image width in pixels */
#define TIFFTAG_IMAGELENGTH             257     /* image height in pixels */
#define TIFFTAG_BITSPERSAMPLE           258     /* bits per channel (sample)*/
#define TIFFTAG_COMPRESSION             259     /*          */
#define TIFFTAG_PHOTOMETRIC             262     /*photometric interpretation*/
#define PHOTOMETRIC_MINISWHITE          0       /* min value is white */
#define PHOTOMETRIC_MINISBLACK          1       /* min value is black */
#define PHOTOMETRIC_RGB                 2       /* RGB color model */
#define PHOTOMETRIC_PALETTE             3       /* color map indexed */
#define PHOTOMETRIC_MASK                4       /* holdout mask */
#define PHOTOMETRIC_DEPTH               32768   /* z-depth data */
#define PHOTOMETRIC_YUV                 6       /* YUV (YCbCr color space */
#define TIFFTAG_IMAGEDESCRIPTION        270     /* info about image */
#define TIFFTAG_MAKE                    271     /* scanner manufacturer name*/
#define TIFFTAG_MODEL                   272     /* scanner model name/number*/
#define TIFFTAG_STRIPOFFSETS            273     /* offsets to data strips */
#define TIFFTAG_ORIENTATION             274     /* +image orientation */
#define ORIENTATION_TOPLEFT             1       /* row 0 top, col 0 lhs */
#define ORIENTATION_TOPRIGHT            2       /* row 0 top, col 0 rhs */
#define ORIENTATION_BOTRIGHT            3       /* row 0 bottom, col 0 rhs */
#define ORIENTATION_BOTLEFT             4       /* row 0 bottom, col 0 lhs */
#define ORIENTATION_LEFTTOP             5       /* row 0 lhs, col 0 top */
#define ORIENTATION_RIGHTTOP            6       /* row 0 rhs, col 0 top */
#define ORIENTATION_RIGHTBOT            7       /* row 0 rhs, col 0 bottom */
#define ORIENTATION_LEFTBOT             8       /* row 0 lhs, col 0 bottom */
#define TIFFTAG_SAMPLESPERPIXEL         277     /* samples per pixel */
#define TIFFTAG_ROWSPERSTRIP            278     /* rows per strip of data */
#define TIFFTAG_STRIPBYTECOUNTS         279     /* bytes counts for strips */
#define TIFFTAG_MINSAMPLEVALUE          280     /* +minimum sample value */
#define TIFFTAG_MAXSAMPLEVALUE          281     /* maximum sample value */
#define TIFFTAG_PLANARCONFIG            284     /* planar configuration */
#define TIFFTAG_XPOSITION               286     /* x page offset of imagelhs*/
#define TIFFTAG_YPOSITION               287     /* y page offset of imagelhs*/
#define TIFFTAG_FREEOFFSETS             288     /*+byte offset to free block*/
#define TIFFTAG_FREEBYTECOUNTS          289     /* +sizes of free blocks */
#define TIFFTAG_DATETIME                306     /* creation date and time */
#define TIFFTAG_ARTIST                  315     /* creator of image */
#define TIFFTAG_HOSTCOMPUTER            316     /* machine where created */
#define TIFFTAG_YUVCOEF                 529     /* YUV (YCbCr) coefficients */
#define TIFFTAG_YUVSUBSAMPLE            530     /* YUV (YCbCr) subsampling */
#define TIFFTAG_YUVPOSITION             531     /* YUV (YCbCr) positioning */
#define TIFFTAG_REFBLACKWHITE           532     /* Reference Black and White*/

/* new error code for sample.c */
#define BUF_ERROR                       -1100
#define BITS_ERROR                      -1101

#if defined(_WINDLL)
#define F_W_ERR wf_w_err
#define F_R_ERR wf_r_err
#define wUINT    UINT
#if defined(C600)
#define wHFILE  int
#else
#define wHFILE   HFILE
#endif  /* C600 */
#define CLOSE   _lclose
#define LSEEK   _llseek
#define WREAD    _lread
#define WWRITE   _lwrite
#define STRLEN  lstrlen
#else
#define F_W_ERR f_w_err
#define F_R_ERR f_r_err
#define wUINT    int
#define wHFILE  int
#define CLOSE   close
#define LSEEK   lseek
#define WREAD    read
#define WWRITE   write
#define STRLEN  strlen
#endif /* _WINDLL */


/*-----------------------------------------------------------------------
-  structure definitions
-------------------------------------------------------------------------
*/

typedef struct {
	ULONG numerator;                /* numerator part of a fraction */
	ULONG denominator;              /* denominator part of a fraction */
} RATIONAL;

typedef struct {
	UWORD tdir_tag;                 /* see below */
	UWORD tdir_type;                /* data type; see below */
	ULONG tdir_count;               /* number of items; length in spec */
	ULONG tdir_offset;              /* byte offset to field data */
} TIFFDirEntry;

struct TIFF_BUF
{
	int fd;         /* file descriptor for I/O      */
	UWORD l;        /* length of this buffer        */
	BYTE rw;        /* read/write mode identifier   */
			/* 0 = r, 1 = w, r/w mode is not supported */
	UWORD sp;       /* space left in this buffer, when read */
			/* it is the data available for read,   */
			/* when write, is the space left for write      */
	UWORD rowsperbuf; /* number of the rows is available in a buffer */
			  /* rowsperbuf = buffer length / byteperrow */
	UWORD bytesperbuf;  /* bytes per buffer = byteperrow * rowsperbuf */
	BYTE  FAR *hp;      /* head pointer of the buffer, fixed    */
	BYTE  FAR *cp;      /* point to the current available segment       */
	ULONG count;        /* the current count of a strip */
	int startrow, endrow;           /* mark rows in buffer */
	ULONG sl;               /* if stripcount >8k,is the space */
				/* left for read */
	ULONG FAR *soffset;     /* array for strip offsets */
	ULONG FAR *scount;      /* array for strip counts  */
#if defined(_WINDLL)            /* WINDOWS need the Mem. Handles */
	HANDLE hbuf;            /* Handle for allocate buffer   */
	HANDLE harray;          /* Handle, allocate offset & counter array*/
	HANDLE soarray;         /* Memory handle for strip offset array */
	HANDLE scarray;         /* Memery handle for strip count array */
#endif /* _WINDLL */
};

typedef struct {
	UWORD   td_subfiletype;                 /* always write 1 */
	ULONG   td_imagewidth, td_imagelength;  /* always write 2,3 */
	UWORD   td_bitspersample;               /* always write 4 */
	UWORD   td_samplesperpixel;             /* always write 5 */
	UWORD   td_compression;                 /* always write 6 */
	ULONG   td_rowsperstrip;                /* always write 7 */
	ULONG   td_minsamplevalue, td_maxsamplevalue;   /* always write 8,9 */
	UWORD   td_photometric;                 /* always write 10 */
	BYTE    FAR *td_artist;
	BYTE    FAR *td_datetime;
	BYTE    FAR *td_hostcomputer;
	BYTE    FAR *td_imagedescription;
	BYTE    FAR *td_make;
	BYTE    FAR *td_model;
	BYTE    FAR *td_software;
	BYTE    FAR *td_pagename;
	ULONG   td_stripsperimage;      
	ULONG   FAR *td_stripoffset;                /* always write 11 */
	ULONG   FAR *td_stripbytecount;             /* always write 12 */
	RATIONAL td_yuvcoef[3];                     /* write only for YUV */
	UWORD   td_yuvsubsample[2];                 /* write only for YUV */
	UWORD   td_yuvposition;                     /* write only for YUV */
	RATIONAL td_refblackwhite[6];               /* write only for YUV */
//#if defined(_WINDLL)
//        HANDLE  htd_imagedescription;
//        HANDLE  htd_make;
//        HANDLE  htd_model; 
//        HANDLE  htd_stripoffset;
//        HANDLE  htd_stripbytecount; 
//        HANDLE  htd_datetime;
//        HANDLE  htd_artist; 
//        HANDLE  htd_hostcomputer;    
//#endif //_WINDLL
} TIFFDirectory;

			/* define tiff file header */
typedef struct {
	UWORD tiff_magic;       /* magic number (defines byte order) */
	UWORD tiff_version;             /* TIFF version number */
	ULONG tiff_diroff;              /* byte offset to first directory */
} TIFFHeader;


typedef struct {
	BYTE    FAR *tif_name;          /* name of open file */
	WORD    tif_fd;                 /* open file descriptor */
	WORD    tif_mode;               /* open mode (O_*) */
	BYTE    tif_fillorder;  /* natural bit fill order for machine */
	INT     tif_numtags;
	TIFFDirectory tif_dir;  /* internal rep of current directory */
	TIFFHeader tif_header;          /* file's header block */
	ULONG   stoffset, stcount;      /* strip's offset & count for FB.C */
	ULONG   begofdat;               /* offset of begin of the data */
	ULONG   byteperrow;
	INT     tif_curstrip;           /* current strip for read/write */
	LONG    tif_curoff;             /* current offset for read/write */
} TIFF;


/*----------------------------------------------------------------------*/
/*-  function prototypes                                                */
/*----------------------------------------------------------------------*/

int MTPROCALL FIORdTIFFHeader(TIFF FAR *);
int MTPROCALL FIOWtTIFFHeader(TIFF FAR *);
void MTPROCALL FIOCloseTIFFFile(TIFF FAR *);
int  PASCAL FIOTypeTIFFHeader(FILE FAR *, TIFF FAR *);
struct TIFF_BUF FAR * MTPROCALL FIOBufRequestTIFF(TIFF FAR *, int, int);
int MTPROCALL FIOBufReadTIFF(TIFF FAR *, struct TIFF_BUF FAR *, BYTE FAR *, int, int);
int MTPROCALL FIOBufWriteTIFF(TIFF FAR *, struct TIFF_BUF FAR *, BYTE FAR *, int);
int MTPROCALL FIOBufReleaseTIFF(TIFF FAR *, struct TIFF_BUF FAR *);
int MTPROCALL FIOEndStripTIFF(TIFF FAR *, struct TIFF_BUF FAR *);


/***** bitmap BMP definitions and types *****/

// DIB bimap definitions are in windows.h
#if !defined(_WINDOWS)

#define BI_RGB  0L
#define BI_RLE8 1L
#define BI_RLE4 2L

typedef unsigned long   DWORD;

typedef struct tagBITMAPINFOHEADER {
	DWORD   biSize;                 // size of BITMAPINFOHEADER
	DWORD   biWidth;                // width in pixels
	DWORD   biHeight;               // height in pixels
	WORD    biPlanes;               // always 1
	WORD    biBitCount;             // color bits per pixel
					// must be 1, 4, 8 or 24
	DWORD   biCompression;          // BI_RGB, BI_RLE8 or BI_RLE4
	DWORD   biSizeImage;            // total bytes in image
	DWORD   biXPelsPerMeter;        // 0, or opt. h res.
	DWORD   biYPelsPerMeter;        // 0, or opt. v res.
	DWORD   biClrUsed;              // normally 0, can set a lower
					// no. colors than biBitCount
	DWORD   biClrImportant;         // normally 0
} BITMAPINFOHEADER;

typedef BITMAPINFOHEADER FAR    *LPBITMAPINFOHEADER;
typedef BITMAPINFOHEADER        *PBITMAPINFOHEADER;

typedef struct tagRGBQUAD {
	BYTE    rgbBlue;                // blue intensity, 0 - 255
	BYTE    rgbGreen;               // green intensity, 0 - 255
	BYTE    rgbRed;                 // red intensity, 0 - 255
	BYTE    rgbReserved;            // reserved, set to zero
} RGBQUAD;

typedef struct tagBITMAPINFO {
	BITMAPINFOHEADER        bmiHeader;
	RGBQUAD                 bmiColors[1];
} BITMAPINFO;

typedef BITMAPINFO FAR  *LPBITMAPINFO;
typedef BITMAPINFO      *PBITMAPINFO;

typedef struct tagBITMAPFILEHEADER {
	WORD    bfType;                 // always equal to 'BM'
//        ULONG    bfType;                 // always equal to 'BM'
	DWORD   bfSize;                 // size of file in DWORDs
	WORD    bfReserved1;            // set to zero
 //       ULONG    bfReserved1;            // set to zero
	WORD    bfReserved2;            // set to zero
//        ULONG    bfReserved2;            // set to zero
	DWORD   bfOffBits;              // byte offset from BITMAPFILEHEADER
					// to bitmap pixel data in the file
} BITMAPFILEHEADER;

typedef BITMAPFILEHEADER FAR    *LPBITMAPFILEHEADER;
typedef BITMAPFILEHEADER        *PBITMAPFILEHEADER;

#endif /* !_WINDOWS */

typedef struct tagBMPHEADER {
	BITMAPFILEHEADER        bmpFileHeader;  // bitmap file header
	BITMAPINFOHEADER        bmpInfoHeader;  // bitmap info header
	WORD                    bmpNumColors;   // number of colors used
	RGBQUAD FAR             *bmpLut;        // color lookup table
#if defined(_WINDOWS)
	HANDLE                  bmpLutHandle;   // memory handle
#endif
} BMPHEADER;

typedef BMPHEADER FAR   *LPBMPHEADER;
typedef BMPHEADER       *PBMPHEADER;

typedef struct tagBMP {
	BYTE FAR        *bmp_name;      // bitmap file name
	WORD            bmp_fd;         // ope file descriptor
	WORD            bmp_mode;       // open mode: READ/WRITE
	BMPHEADER       bmp_header;     // bitmap header
} BMP;


/*----------------------------------------------------------------------*/
/*-  function prototypes                                                */
/*----------------------------------------------------------------------*/

int MTPROCALL FIORdBMPHeader(BMP FAR *);
int MTPROCALL FIOWtBMPHeader(BMP FAR *);
void MTPROCALL FIOCloseBMPFile(BMP FAR *);
int MTPROCALL FIOTypeBMPHeader(FILE FAR *, BMP FAR *);

#endif /* FIO_H */

/*------------------------------ end of fio.h ----------------------------*/
